
function Remove-ipxExchangeLogfiles {
    <#
        .Synopsis
            Dummy Syn
        .Description
            Dummy Desc
        .Parameter Dummy
            Dummy Par
        .Example 
            Dummy
        .Inputs
            Dummy In
        .Outputs
            Dummy Out
        .Notes
            Dummy Notes
        .Component
            Dummy Com
    #>

    param (
    [Parameter(HelpMessage='Minimales Alter der Logfiles die entfernt werden.')]
    [string]$Days=0,
    [Parameter(HelpMessage='Welcher Logginggrad soll ausgegeben werden.')]
    [string]$Loglevel=0,
    [Parameter(HelpMessage='Grundinstallationspfad des Exchange Servers')]
    [string]$Rootpath
    )

    <#
    $IISLogPath="C:\inetpub\logs\LogFiles\" #.log
    $ExchangeLoggingPath="C:\Program Files\Microsoft\Exchange Server\V15\Logging\" #.log
    $ETLLoggingPath="C:\Program Files\Microsoft\Exchange Server\V15\Bin\Search\Ceres\Diagnostics\ETLTraces\" #.etl
    $ETLLoggingPath2="C:\Program Files\Microsoft\Exchange Server\V15\Bin\Search\Ceres\Diagnostics\Logs" #.log
    #>

    $IISLogPath="C:\inetpub\logs\LogFiles\" #.log
    $ExchangeLoggingPath=$Rootpath + "\Logging\" #.log
    $ETLLoggingPath=$Rootpath + "\Bin\Search\Ceres\Diagnostics\ETLTraces\" #.etl
    $ETLLoggingPath2=$Rootpath + "\Bin\Search\Ceres\Diagnostics\Logs" #.log

    Function CleanLogfiles($TargetFolder)
    {
        if (Test-Path $TargetFolder) {
            $LastWrite = (Get-Date).AddDays(-$days)
            
            if ($Loglevel -ge "1" ) { Write-Host "Aktuell in: $TargetFolder" }
            $Files = (Get-ChildItem $TargetFolder -Include *.log,*.etl -Recurse | Where-Object {$_.LastWriteTime -le "$LastWrite"})
            if ($Loglevel -ge "1" ) { Write-Host "Dateien ermittelt, jetzt wird gelscht." -ForegroundColor Red }
            foreach ($File in $Files) {
                if ($Loglevel -eq "2") { Write-Host "Lsche: " $File.Fullname }

                
                Remove-Item -Path $File.FullName -Force -ErrorAction SilentlyContinue
            }
           }
    Else {
        Write-Host "The folder $TargetFolder doesn't exist! Check the folder path!" -ForegroundColor "white"
        }
    }
    CleanLogfiles($IISLogPath)
    CleanLogfiles($ExchangeLoggingPath)
    CleanLogfiles($ETLLoggingPath)
    CleanLogfiles($ETLLoggingPath2)

}

function Convert-ipxSynologyCertificateToPfx {
    <#
        .Synopsis
            Konvertiert die Zertifikatdateien vom Synology NAS in eine PFX-Version
        .Description
            Im Synology NAS wird zB das Lets Encrypt Zertifikat abgerufen und dann exportiert. Das entstandene zip-Archiv wird entpackt,
            es liegen dann die einzelnen *.pem Dateien dort. ber OpenSSL wird anschlieend die PFX-Datei aus den drei einzelnen Dateien konvertiert
            Im Programmablauf wird nach dem entpackten Ordner und dem Namen fr das Zertifikat gefragt (ohne Dateiendung!).
            Das Ergebnis wird im selben Ordner wie die Quellen liegen.
            Die Routine luft nicht in der Powershell ISE!
        .Inputs
            Synology Exported Certificate As PEM
        .Outputs
            Converted Certificate as PFX
        .Notes
            Version: 1.0.0.0
            Datum: 06.07.2018
            Doesnt work in Powershell ISE
        .Component
            GnuWin OpenSSL 0.9.7
    #>

    param (

    )
    $OpenSSLExecute = 'C:\Program Files (x86)\GnuWin32\bin\openssl.exe'
    If ( -not ( Test-Path $OpenSSLExecute)) {
        Write-Host "OpenSSL wurde nicht im Standardpfad gefunden, die Konvetierung kann nicht erfolgen. :(" -ForegroundColor Red
        break;
    }
    
    $SynologyCertificateFolder = (Get-ipxFoldername -description "Den entpackten Synology Zertifikatordner angeben")
    If ( -not ( Test-Path $SynologyCertificateFolder\cert.pem )) {
        Write-Host "cert.pem fehlt im Ordner!" -ForegroundColor Red
        break;
    }
    If ( -not ( Test-Path $SynologyCertificateFolder\chain.pem )) {
        Write-Host "chain.pem fehlt im Ordner!" -ForegroundColor Red
        break;
    }
    If ( -not ( Test-Path $SynologyCertificateFolder\privkey.pem )) {
        Write-Host "privkey.pem fehlt im Ordner!" -ForegroundColor Red
        break;
    }
    Write-Host "Alles geprft, los gehts! :)" -ForegroundColor Green
    $CertificateName = Read-Host -Prompt "Dateiname des neuen Zertifikats eingeben"
    $Type = "pkcs12"

    & $OpenSSLExecute pkcs12 -export -out $SynologyCertificateFolder\$CertificateName.pfx -inkey $SynologyCertificateFolder\privkey.pem -in $SynologyCertificateFolder\cert.pem -certfile $SynologyCertificateFolder\chain.pem

}
