﻿function Test-ipxPSCX {
    <#
        .Synopsis
            Testet auf die Existenz der Powershell Community Extension
        .Description
            Wenn das Modul verfügbar ist wird $true zurückgegeben, sonst $false
        .Inputs
            None
        Outputs
            System.Boolean
        .Notes
            Version: 1.0.0.0
            Datum: 1.3.2015
        .Component
            None
    #>
if (Get-Module -ListAvailable | Where-Object { $_.Name -match "PSCX" } ) {
    return $true
}
else { return $false }
}

function Test-ipxISE{
    <#
        .Synopsis
            Testet die aktuelle Powershell Session auf Administratorrechte
        .Description
            Wenn im Powershellwindows "Administrator:" steht gibt die Funktion $true zurück, sonst $false
        .Inputs
            None
        Outputs
            System.Boolean
        .Notes
            Version: 1.0.0.0
            Datum: 1.3.2015
        .Component
            None
    #>

    if( $psISE -ne $null ) {
        return $true;
    }
    else {
        return $false;
    }        
}

function Get-ipx7zip {
    <#
        .Synopsis
            Gibt den vollqualifierten Dateipfad zur integrierten 7za.exe zurück.
        .Description
            Es wird der Modulpfad ausgelesen und die 7za.exe ermittelt.
            Der Rückgabewert ist ein String (.Fullname) zur exe-Datei, damit sie von anderen Cmdlets aufgerufen werden kann.
        .Example 
            Get-ipx7zip
            Gibt den vollen (UNC-)Pfad zurück, z.B. \\Fileserver01\Daten\...\7za.exe
        .Inputs
            Nothing
        .Outputs
            System.String Fullname
        .Notes
            Version: 1.0.0.0
            Datum: 9.1.2017
        .Component
            exe\7za.exe
    #>

    $zipexe = Get-Item -Path $MyModuls\Modules\ipx\exe\7za.exe
    return $zipexe.FullName
}

function Get-ipxCommandlist {
<#
    .Synopsis
        Cmdlet-Übersicht eines oder aller Module ausgeben.
    .Description
        Gibt für ein anzugebenes Modul oder alle Module die Cmdlets (Befehle) und deren Beschreibung aus.
        Export als CSV oder Gridview muss selbst als Pipeline implementiert werden.
    .Example
        Get-ipxCommandlist
        Listet von allen verfügbaren Modulen die Befehle und deren Kurzbeschreibung
    .Example
        Get-ipxCommandlist -modulename ipx
        Listet alle Cmdlets (Befehle) und deren Kurzbeschreibung vom Modul ipx
    .Inputs
        None 
    .Outputs
        System.Array
    .Notes
        Version: 1.0.0.0
        Datum: 23.2.2015
    .Component
        None
#>
    param (
    [Parameter(HelpMessage='Modulname für den die Cmdlets ermittelt werden sollen',ValueFromPipeline=$true,ValueFromPipelineByPropertyName=$true)]
    [string]$Name = ''
    )
    process {
    if ( $Name -eq '' ) { $moduleset = Get-Module -ListAvailable }
        else { $moduleset = Get-Module -Name $Name }

        $arr = @()
        foreach ( $module in $moduleset ) {
            Write-Progress -Activity $module.Name
        
            Get-Command -Module $module.Name | 
                % {
                    [string]$synopsis = (Get-Help -Name $_.Name).Synopsis
                    if ( $synopsis.Contains("[")) { $synopsis = "n.a."}

                    $obj = New-Object PSObject
                    $obj | Add-Member NoteProperty Module $_.ModuleName
                    $obj | Add-Member NoteProperty Cmdlet $_.Name
                    $obj | Add-Member NoteProperty Synopsis $synopsis

                    $arr += $obj
                }
            }
        $arr
    }   
}

function Convert-ipxToMB {
<#
    .Synopsis
        Umwandlung einer Zahl in Megabyte und Rundung auf 2 Kommastellen
    .Description
        Umwandlung einer Zahl in Megabyte, entweder über die Pipeline oder direkt über den Parameter size.
        Die Rundung kann angepasst werden.
    .Parameter size
        Übergibt die umzurechnende Zahl
    .Parameter precision
        Genauigkeit bei der Rundung, Vorgabewert 2
    .Example 
        Get-ipxSize -path C: | Convert-ipxToMB
        Ermittelt die Dateigrößen im laufwerk C und gibt sie als Megabyte mit der Genauigkeit 2 aus.
    .Inputs
        System.ValueType size und precision
    .Outputs
        System.Double
    .Notes
        Version 1.0.0.1
        Datum 23.2.2015
    .Component
        None
#>
    param (
        [Parameter(Mandatory=$true,HelpMessage='Zu wandelnde Zahl',ValueFromPipeline=$true,ValueFromPipelineByPropertyName=$true)]
        $size,
        [Parameter(HelpMessage='Genauigkeit')]
        [Byte]$precision = 2
    )
    process {
        $newsize = $size / 1MB
        $newsize = [math]::Round($newsize,$precision)
        $newsize
    }
}

function Convert-ipxToGB {
<#
    .Synopsis
        Umwandlung einer Zahl in Gigabyte und Rundung auf 2 Kommastellen
    .Description
        Umwandlung einer Zahl in Gigabyte, entweder über die Pipeline oder direkt über den Parameter size.
        Die Rundung kann angepasst werden.
    .Parameter size
        Übergibt die umzurechnende Zahl
    .Parameter precision
        Genauigkeit bei der Rundung, Vorgabewert 2
    .Example 
        Get-ipxSize -path C: | Convert-ipxToGB
        Ermittelt die Dateigrößen im laufwerk C und gibt sie als Gigabyte mit der Genauigkeit 2 aus.
    .Inputs
        System.ValueType size und precision
    .Outputs
        System.Double
    .Notes
        Version 1.0.0.1
        Datum 23.2.2015
    .Component
        None
#>       
    param (
        [Parameter(Mandatory=$true,HelpMessage='Zu wandelnde Zahl',ValueFromPipeline=$true,ValueFromPipelineByPropertyName=$true)]
        $size,
        [Parameter(HelpMessage='Genauigkeit')]
        [Byte]$precision = 2
    )
    process {
        $newsize = $size / 1GB
        $newsize = [math]::Round($newsize,$precision)
        $newsize
    }
}

function Convert-ipxTokKB {
<#
    .Synopsis
        Umwandlung einer Zahl in Kilobyte und Rundung auf 2 Kommastellen
    .Description
        Umwandlung einer Zahl in Kilobyte, entweder über die Pipeline oder direkt über den Parameter size.
        Die Rundung kann angepasst werden.
    .Parameter size
        Übergibt die umzurechnende Zahl
    .Parameter precision
        Genauigkeit bei der Rundung, Vorgabewert 2
    .Example 
        Get-ipxSize -path C: | Convert-ipxToKB
        Ermittelt die Dateigrößen im laufwerk C und gibt sie als Kilobyte mit der Genauigkeit 2 aus.
    .Inputs
        System.ValueType size und precision
    .Outputs
        System.Double
    .Notes
        Version 1.0.0.1
        Datum 23.2.2015
    .Component
        None
#>
    param (
        [Parameter(Mandatory=$true,HelpMessage='Zu wandelnde Zahl',ValueFromPipeline=$true,ValueFromPipelineByPropertyName=$true)]
        $size,
        [Parameter(HelpMessage='Genauigkeit')]
        [Byte]$precision = 2
    )
    process {
        $newsize = $size / 1KB
        $newsize = [math]::Round($newsize,$precision)
        $newsize
    }
}

function Convert-ipxToTB {
<#
    .Synopsis
        Umwandlung einer Zahl in Terabyte und Rundung auf 2 Kommastellen
    .Description
        Umwandlung einer Zahl in Terabyte, entweder über die Pipeline oder direkt über den Parameter size.
        Die Rundung kann angepasst werden.
    .Parameter size
        Übergibt die umzurechnende Zahl
    .Parameter precision
        Genauigkeit bei der Rundung, Vorgabewert 2
    .Example 
        Get-ipxSize -path C: | Convert-ipxToTB
        Ermittelt die Dateigrößen im laufwerk C und gibt sie als Terabyte mit der Genauigkeit 2 aus.
    .Inputs
        System.ValueType size und precision
    .Outputs
        System.Double
    .Notes
        Version 1.0.0.1
        Datum 23.2.2015
    .Component
        None
#>
    param (
        [Parameter(Mandatory=$true,HelpMessage='Zu wandelnde Zahl',ValueFromPipeline=$true,ValueFromPipelineByPropertyName=$true)]
        $size,
        [Parameter(HelpMessage='Genauigkeit')]
        [Byte]$precision = 2
    )
    process {
        $newsize = $size / 1TB
        $newsize = [math]::Round($newsize,$precision)
        $newsize
    }
}

function Export-ipxCredentialAsXML {
<#
    .Synopsis
        Fragt die Benutzerdaten ab und exportiert sie in eine verschlüsselte XML-Datei.
    .Description
        Nach Aufruf des Cmdlets werden der Benutzername und das Passwort abgefragt.
        Sollen Domänenbenutzer benutzt werden, ist die vollständige Eingabe Domäne\Benutzer notwendig.
        Der XML-Dateiname wird durch .Split("\") aus dem Benutzernamen extrahiert.
    .Parameter path
        Gibt den Pfad der zu speichernden XML-Datei an, der Dateiname wird aus dem Benutzernamen erzeugt.
    .Example
        Export-ipxCredentialAsXML -path $env:userprofile
        Fragt die Benutzerdaten ab und legt sie im eigenen Benutzerprofil ab.
    .Inputs
        System.IO.DirectoryInfo path als String Ordnerpfad
    .Outputs
        System.IO.FileInfo Credential *.xml
    .Notes
        Version: 1.0.0.0
        Datum 23.2.2015
    .Component
        None
#>
param (
    [Parameter(Mandatory=$true,HelpMessage='Pfad wo die XML-Datei abgelegt wird')]
    [string]$path
)
$username = Read-Host "Benutzername"
$password = Read-Host "Passwort" -AsSecureString
$credential = New-Object System.Management.Automation.PSCredential($username, $password)
$user = ($username).Split("\")
$file = $user[-1]
$credential | Export-Clixml -Path "$path\$file.xml"

}

function Import-ipxCredentialFromXML {
<#
    .Synopsis
        Import Benutzername und Password aus einer verschlüsselten XML-Datei
    .Description
        Beim Aufruf des Cmdlets wird die XML-Datei abgefragt und anschließend eingelesen.
        Der Import kann in eine Variable oder für Programme mit Sicherheitsabfrage verwendet werden.
    .Parameter file
        Gibt den vollständigen Pfad zur XML-Datei an.
    .Example
        Import-ipxCredentialFromXML -file $env:userprofile\user.xml
        Import die Daten aus der user.xml die im Benutzerverzeichnis liegt.
    .Example
        $credential = Import-ipxCredentialFromXML -file $env:userprofile\user.xml
        Import die Daten aus der user.xml und schreibt sie in die Variable $credential.
        $credential.UserName und $credential.Password kann für weitere Aktionen genutzt werden.
    .Example
        Start-Process $pshome\powershell.exe -credential (Import-ipxCredentialFromXML -file $env:userprofile\user.xml)
        Startet eine neue Powershell-Instanz mit den Rechten die in der importierten XML-Datei hinterlegt sind.
        !! Wird %homepath% des Benutzers über das Domänenkonto umgestellt, kann der Aufruf fehlschlagen !!
    .Inputs
        System.IO.FileInfo Credential *.xml
    .Outputs
        System.Management.Automation.PSCredential
    .Notes
        Version: 1.0.0.0
        Datum 23.2.2015
    .Component
        None
#>
param (
    [Parameter(Mandatory=$true,HelpMessage='Pfad wo die XML-Datei abgelegt wird')]
    [string]$file
)
$credential = Import-Clixml $file
$username = $credential.username
$password = $credential.password
New-Object System.Management.Automation.PSCredential($username, $password)
}

function Test-ipxAdminRights{
    <#
        .Synopsis
            Testet die aktuelle Powershell Session auf Administratorrechte
        .Description
            Wenn im Powershellwindows "Administrator:" steht gibt die Funktion $true zurück, sonst $false
        .Example 
            if ( !(Test-ipxAdminrights) ) { Write-Host "Kein Admin"}
            Testet ob die aktuelle Session Adminrechte hat, und wenn nicht, wird der Text ausgegeben.
        .Inputs
            None
        .Outputs
            System.Boolean
        .Notes
            Version: 1.0.0.0
            Datum: 23.2.2015
        .Component
            None
    #>

    if( $host.UI.RawUI.WindowTitle.StartsWith( "Admin" ) ) {
        return $true;
    }
    else {
        return $false;
    }       
}

function Invoke-ipxAdminRights{
<#
    .Synopsis
        Testet den aktuellen Benutzer auf die Zugehörigkeit der Adminbenutzer und hebt die Rechte in einer neuen Powershell-Sitzung an.
    .Parameter commandlist
        Gibt ein Cmdlet oder Skript an was dann mit erhöhten Rechten ausgeführt wird. Optional können Cmdlets oder Skripte übergeben geben.
    .Parameter admingroup
        Gibt optional eine Administratorengruppe an derer der Benutzer angehört, Relevanz vor allem für Domänen bei der Benutzer über OU und GPO verwaltet werden. Vorgabewert ist "Administratoren".
    .Notes
        Version: 1.0.0.0
        Datum: 14.7.2014
    .Example
        Invoke-ipxAdminrights
        Startet eine neue Powershell Instanz mit erhöhten Rechten.
        Der Benutzer muss direkt in der lokalen Administratorgruppe sein, sonst wird die Anfrage abgelehnt.
    .Example
        Invoke-ipxAdminrights -admingroup "lokale Admins"
        Startet eine neue Powershell Instanz mit erhöhten Rechten, wobei der Benutzer der Domänengruppe "lokale Admins"
        angehört die per Gruppenrichtlinie dem Computer zugeordnet wurde.
    .Example
        Invoke-ipxAdminrights -commandlist "Set-ExecutionPolicy unrestricted"
        Startet eine neue Powershell Instanz mit erhöhten Rechten und setzt die Ausführungsrichtlinie auf uneingeschränkt.
        Danach schließt sich das Fenster sofort ohne Rückmeldung
    .Example
        Invoke-ipxAdminrights -commandlist "Get-ExecutionPolicy; Read-Host"
        Startet eine neue Powershell Instanz mit erhöhten Rechten und fragt die Ausführungsrichtlinie ab.
        Mit Semikolon getrennt wird der nächste Befehl übergeben, Read-Host hält das Fenster geöffnet bis eine Eingabe getätigt wird.
    .Inputs
        System.ValueType.String commandlist und admingroup
    .Outputs
        None
    .Notes
        Version: 1.0.0.0
        Datum: 23.2.2015
    .Component
        None
#>
param (
    [Parameter(HelpMessage='optionale Argumente wie Befehle')]
    [string]$commandlist,
    [Parameter(ParameterSetName='local',HelpMessage='Angabe der abweichen Domänen-Administratorgruppe')]
    [string]$admingroup="Administratoren"
)
    $currentPrincipal = New-Object Security.Principal.WindowsPrincipal( [Security.Principal.WindowsIdentity]::GetCurrent() ) 
    if ($currentPrincipal.IsInRole( $admingroup )) { 
        if ( $commandlist -eq "" ) {
        Start-Process $pshome\powershell.exe -Verb runas
        }
            else {
            Start-Process $pshome\powershell.exe -Verb runas -ArgumentList $commandlist
            }
        } 
    else { 
        Write-Host "Der aktuelle Benutzer gehört nicht der Administratorengruppe an." -ForegroundColor Red
        break; 
        }
}

function Invoke-ipxAdminRightsFromCredential{
<#
    .Synopsis
        Fragt die Benutzerdaten eines berechtigten Administrators ab und startet eine neue Powershell-Instanz mit diesen Rechten.
    .Parameter commandlist
        Gibt ein Cmdlet oder Skript an was dann mit erhöhten Rechten ausgeführt wird.
    .Notes
        Version: 1.0.0.0
        Datum: 14.7.2014
    .Example
        Invoke-AdminrightsFromCredential
        Startet eine neue Powershell Instanz mit Abfrage der Administratordaten.
        Bei Domänenaccounts muss zur sauberen Auflösung Domäne\user eingetragen werden.
    .Example
        Invoke-AdminrightsFromCredential -commandlist "Set-ExecutionPolicy unrestricted"
        Startet eine neue Powershell Instanz nach Abfrage der Administratordaten mit erhöhten Rechten und setzt die Ausführungsrichtlinie auf ungeprüft.
        Danach schließt sich das Fenster sofort ohne Rückmeldung
    .Inputs
        System.String commandlist
    .Outputs
        None
    .Notes
        Version: 1.0.0.0
        Datum: 23.2.2015
    .Component
        None
#>
param (
    [Parameter(Mandatory=$false,HelpMessage='optionale Argumente wie Befehle')]
    [string]$commandlist
)
    if ( $commandlist -eq "" ) {
        Start-Process $pshome\powershell.exe -Credential (Get-Credential)
        }
        else {
        Start-Process $pshome\powershell.exe -Credential (Get-Credential) -ArgumentList $commandlist
        }
      
}

function Invoke-ipxAdminRightsFromXML{
<#
    .Synopsis
        Startet eine neue Powershell-Instanz auf Grundlage einer XML mit den verschlüsselten Anmeldedaten des dort hinterlegten Benutzers.
    .Parameter commandlist
        Gibt ein Cmdlet oder Skript an was dann mit erhöhten Rechten ausgeführt wird. Das Cmdlet kann unbeaufsichtigt für Routineaufgaben benutzt werden.
    .Parameter xml
        Gibt den Pfad zur XML-Datei mit den Anmeldedaten an, Bsp: C:\Users\username\security.xml
        Der Parameter muss zwingend angegeben werden.
    .Notes
        Version: 1.0.0.0
        Datum: 14.7.2014
    .Example
        Invoke-ipxAdminrightsFromXML -xml C:\Users\username\security.xml
        Nach dem Einlesen der XML-Datei wird eine neue Powershell-Instanz mit den neuen Rechten gestartet.
        !! Wird %homepath% des Benutzers über das Domänenkonto umgestellt, kann der Aufruf fehlschlagen !!
    .Example
        Invoke-ipxAdminrightsFromXML -xml C:\Users\username\security.xml -commandlist 'Read-Host "Eingabe"'
        Nach dem Einlesen der XML-Datei wird eine neue Powershell-Instanz mit den neuen Rechten gestartet.
        Das übergebene Read-Host Cmdlet wird sofort ausgeführt und hält das Fenster geöffnet bis eine Taste gedrückt wird.
        !! Wird %homepath% des Benutzers über das Domänenkonto umgestellt, kann der Aufruf fehlschlagen !!
    .Inputs
        System.String commandlist
        System.IO.FileInfo Credential *.xml
    .Outputs
        None
    .Notes
        Version: 1.0.0.0
        Datum: 23.2.2015
    .Component
        None
#>
param (
    [Parameter(Mandatory=$false,HelpMessage='optionale Argumente wie Befehle')]
    [string]$commandlist,
    [Parameter(Mandatory=$true,HelpMessage='XML-File mit abgelegter Authentifizierung')]
    [string]$xml
)
    if ( $commandlist -eq "" ) {
        Start-Process $pshome\powershell.exe -Credential (Import-CredentialFromXML -file $xml)
        }
        else {
        Start-Process $pshome\powershell.exe -Credential (Import-CredentialFromXML -file $xml) -ArgumentList $commandlist
        }
      
}

function Get-ipxFoldername {
<#
    .Synopsis
        Windows Dateidialog für Abfrage eines Ordners
    .Parameter rootpath
        Gibt das Verzeichnis an mit dem der Dialog initialisiert wird
    .Parameter description
        Gibt dem Benutzer im Dialogfeld einen Hinweistext was zu tun ist
    .Example
        Get-ipxFoldername -rootpath C:\Users\User -description Bitte Ordner wählen
        Öffnet den Dialog im Benutzerverzeichnis und gibt den Hinweistext mit an.
    .Inputs
        System.String rootpath und description
    .Outputs
        System.String
    .Notes
        Version: 1.0.0.0
        Datum: 23.2.2015
    .Component
        System.Windows.Forms
#>
    param (
        [Parameter(HelpMessage='Pfad in dem der Dialog geöffnet wird')]
        [string]$rootpath,
        [Parameter(HelpMessage='Hilfstext im Dialogfeld')]
        [string]$description
    )
    [void] [System.Reflection.Assembly]::LoadWithPartialName("System.Windows.Forms")
    $filefinder = New-Object System.Windows.Forms.FolderBrowserDialog
    $filefinder.SelectedPath = $rootpath
    $filefinder.Description = "$description"
    $filefinder.ShowNewFolderButton = $false

    if($filefinder.ShowDialog() -eq 'OK'){ 
        $filefinder.SelectedPath;
    }
    else {
    $null;
    }
}

function Set-ipxFoldername {
<#
    .Synopsis
        Windows Dateidialog für das Festlegen eines Ordners in den geschrieben werden soll
    .Parameter rootpath
        Gibt das Verzeichnis an mit dem der Dialog initialisiert wird
    .Parameter description
        Gibt dem Benutzer im Dialogfeld einen Hinweistext was zu tun ist
    .Example
        Set-ipxFoldername -rootpath C:\Users\User -description Bitte Ordner wählen
        Öffnet den Dialog im Benutzerverzeichnis und gibt den Hinweistext mit an.
    .Inputs
        System.String rootpath und description
    .Outputs
        System.String
    .Notes
        Version: 1.0.0.0
        Datum: 23.2.2015
    .Component
        System.Windows.Forms
#>
    param (
        [Parameter(HelpMessage='Pfad in dem der Dialog geöffnet wird')]
        [string]$rootpath,
        [Parameter(HelpMessage='Hilfstext im Dialogfeld')]
        [string]$description
    )
    [void] [System.Reflection.Assembly]::LoadWithPartialName("System.Windows.Forms")
    $filefinder = New-Object System.Windows.Forms.FolderBrowserDialog
    $filefinder.SelectedPath = $rootpath
    $filefinder.Description = "$description"
    $filefinder.ShowNewFolderButton = $true
    if($filefinder.ShowDialog() -eq 'OK'){ 
        $filefinder.SelectedPath;
    }
    else {
    $null;
    }
}

function Get-ipxFilename {
<#
    .Synopsis
        Windows Dateidialog für Abfrage einer zu lesenden Datei
    .Parameter rootpath
        Gibt das Verzeichnis an mit dem der Dialog initialisiert wird
    .Parameter title
        Gibt den Titel für den Fensterkopf an
    .Parameter filter
        Gibt eine Filterauswahl für Dateiendungen an
    .Example
        Get-ipxFilename -rootpath \\server\ini -title 'csv-Datei auswählen' -filter "CSV|*.csv"
        Gibt den vollständigen Dateinamen (FullName) einer CSV-Datei zurück, wobei ein Netzwerkpfad vorgeschlagen wird.
    .Inputs
        System.String rootpath und title und filter
    .Outputs
        System.String
    .Notes
        Version: 1.0.0.0
        Datum: 23.2.2015
    .Component
        System.Windows.Forms
#>
    param (
        [Parameter(Mandatory=$true,HelpMessage='Pfad in dem der Dialog geöffnet wird')]
        [string]$rootpath,
        [Parameter(HelpMessage='Dialogtitel in Kopfzeile')]
        [string]$title = 'Öffnen',
        [Parameter(HelpMessage='Dateifilter')]
        [string]$filter = ''
    )
    [void] [System.Reflection.Assembly]::LoadWithPartialName("System.Windows.Forms")
    
    $filefinder = New-Object System.Windows.Forms.OpenFileDialog
    $filefinder.InitialDirectory = $rootpath
    $filefinder.Title = $title
    $filefinder.Filter = $filter
    if($filefinder.ShowDialog() -eq 'OK'){ 
        $filefinder.FileName;
    }
    else {
    $null;
    }
}

function Set-ipxFilename {
<#
    .Synopsis
        Windows Dateidialog für Speichern einer Datei
    .Parameter rootpath
        Gibt das Verzeichnis an mit dem der Dialog initialisiert wird
    .Parameter title
        Gibt den Titel für den Fensterkopf an
    .Parameter filter
        Gibt eine Filterauswahl für Dateiendungen an
    .Example
        Set-ipxFilename -rootpath \\server\logs -title 'logfile als csv abspeichern' -filter "CSV|*.csv"
        Gibt den vollständigen Dateinamen (FullName) eine CSV-Datei zurück die vorhanden oder neu angelegt wird.
        Der Dialog wird mit einem Netzwerkpfad initialisiert.
    .Inputs
        System.String rootpath und title und filter
    .Outputs
        System.String
    .Notes
        Version: 1.0.0.0
        Datum: 23.2.2015
    .Component
        System.Windows.Forms
#>
    param (
        [Parameter(Mandatory=$true,HelpMessage='Pfad in dem der Dialog geöffnet wird')]
        [string]$rootpath,
        [Parameter(HelpMessage='Dialogtitel in Kopfzeile')]
        [string]$title = 'Speichern',
        [Parameter(HelpMessage='Dateifilter')]
        [string]$filter = ''
    )
    [void] [System.Reflection.Assembly]::LoadWithPartialName("System.Windows.Forms")
    
    $filefinder = New-Object System.Windows.Forms.SaveFileDialog
    $filefinder.InitialDirectory = $rootpath
    $filefinder.Title = $title
    $filefinder.Filter = $filter
    if($filefinder.ShowDialog() -eq 'OK'){ 
        $filefinder.FileName;
    }
    else {
    $null;
    }
}

# eingebaute Funktion, um dieses Modul
# unter einem neuen Namen zu verwenden:
Function New-ipxCustomModule {
<#
    .Synopsis
        Erstellt ein neues Powershell Modul
    .Description
        Angepasaste Routine um neues PowerShell Modul anzulegen und die Grundlagen aus diesem zu übernehmen.
#>
  param (
    [Parameter(Mandatory=$true)]
    $Name,
    $Description,
    $Copyright,
    $Author,
    $Company
  )

  # Pfadname für neues Modul
  $Path = Join-Path -Path (Split-Path -Path $profile) -ChildPath "Modules\$Name"

  if (Test-Path -Path $Path) {
    Write-Warning "Ein Modul namens '$Name' existiert bereits."
    break
  }

  $null = New-Item -Path $Path -ItemType Directory -Force

  # eigenes Modul dorthin kopieren
  $ModuleName = Split-Path -Path $PSScriptRoot -Leaf
  Copy-Item -Path $PSScriptRoot\ModuleLoader.psm1 -Destination $Path

  # Manifestdatei anpassen
  $hash = Invoke-Expression( (Get-Content "$PSScriptRoot\$ModuleName.psd1" | Out-String) )
  $hash.Remove('FunctionsToExport')
  $hash.Remove('CmdletsToExport')
  $hash.Remove('VariablesToExport')
  $hash.Remove('AliasesToExport')
  $hash.GUID = [system.guid]::NewGuid().ToString()
  $hash.Author = $Author
  $hash.CompanyName = $Company
  $hash.Copyright = $Copyright
  $hash.Description = $Description
  $hash.ModuleVersion = '1.0.0.0'
  New-ModuleManifest -Path "$Path\$Name.psd1" @hash

  # Modulordner im Explorer öffnen
  Invoke-Item -Path $Path
}

function Import-ipxXAML {
    <#
        .Synopsis
            Importiert eine WPF XAML Oberflächendatei.
        .Description
            Läd das Presentation Framwork und läd anschließend die angegebene xaml-Datei.
            Nach dem Import werden aus allen Elementnamen Variablen erzeugt um sie im Code anzusprechen.
            Das Cmdlet wird ausschließlich von ipx-Cmdlets genutzt die auch eine GUI besitzen.
        .Parameter Path
            Pfad zur xaml-Datei.
        .Inputs
            System.String xaml-Datei
        .Outputs
            WPF-Fenster, Variablen aller Steuerelemente
        .Notes
            Version: 1.0.0.0
            Datum: 9.1.2017
        .Component
            \xaml\*.xaml
    #>

    [CmdletBinding()]
    Param(
        [Parameter(Mandatory=$True,Position=1)]
        [string]$Path
    )

    [xml]$Global:xmlWPF = Get-Content -Path $Path
    #Add WPF and Windows Forms assemblies

    try{
        Add-Type -AssemblyName PresentationCore,PresentationFramework,WindowsBase,system.windows.forms
    }
    catch {
        Throw "Failed to load Windows Presentation Framework assemblies."
    }

    #Create the XAML reader using a new XML node reader
    $Global:xamGUI = [Windows.Markup.XamlReader]::Load((new-object System.Xml.XmlNodeReader $xmlWPF))

 

    #Create hooks to each named object in the XAML
    $xmlWPF.SelectNodes("//*[@Name]") | %{
        Set-Variable -Name ($_.Name) -Value $xamGUI.FindName($_.Name) -Scope Global
    }
}

function Export-ipxCSV {
    <#
        .Synopsis
            Exportiert Daten als Semikolon-separierte csv.
        .Description
            Verwendet das integrierte Export-CSV Cmdlet, setzt jedoch fest UTF8 und das Semikolon als Vorgabe.
            Der Dateimodus ist immer -Append um die Pipeline zu unterstützen.
            Das Cmdlet wird von allen ipx-Cmdlets mit CSV-Export (Parameter) benutzt.
        .Parameter InputObject
            Der übergebene Wert oder Datensatz, welcher in die Datei geschrieben wird. Stellt die Pipeline-Variable dar.
        .Parameter Path
            Der vollqualifizierte Dateipfad zur *.csv.
        .Example 
            Get-Childitem -Path D:\MeinOrdner | export-ipxCSV -Path D:\Meinordner.csv
            Ermittelt alle Dateien und Ordner im angegebenen Pfad und exportiert die Informationen in die CSV.
        .Inputs
            InputObject Many
            Path System.String Pfad zur *.csv
        .Outputs
            System.IO.File *.csv
        .Notes
            Version: 1.0.0.0
            Datum: 9.1.2017
        .Component
            None
    #>

    Param(
        [Parameter(Mandatory=$True,HelpMessage='Datenobjekt zum Abspeichern',ValueFromPipeline=$true,ValueFromPipelineByPropertyName=$true)]
        $InputObject,
        [Parameter(Mandatory=$True,HelpMessage='Pfad zur CSV-Datei')]
        [string]$Path
    )

    process {
        Export-CSV -InputObject $InputObject -NoTypeInformation -Encoding UTF8 -LiteralPath $Path -Delimiter ";" -Append
    }
}